@@ -23,116 +23,82 @@
  ***************************************************************************/
 #include "tool_setup.h"
 
-#define ENABLE_CURLX_PRINTF
-/* use our own printf() functions */
-#include "curlx.h"
-#include "dynbuf.h"
+#ifndef CURL_DISABLE_IPFS
 
 #include "tool_cfgable.h"
 #include "tool_msgs.h"
 #include "tool_ipfs.h"
-
 #include "memdebug.h" /* keep this as LAST include */
 
-/* ensure input ends in slash */
-static CURLcode ensure_trailing_slash(char **input)
+/* input string ends in slash? */
+static bool has_trailing_slash(const char *input)
 {
-  if(*input && **input) {
-    size_t len = strlen(*input);
-    if(((*input)[len - 1] != '/')) {
-      struct curlx_dynbuf dyn;
-      curlx_dyn_init(&dyn, len + 2);
-
-      if(curlx_dyn_addn(&dyn, *input, len)) {
-        Curl_safefree(*input);
-        return CURLE_OUT_OF_MEMORY;
-      }
-
-      Curl_safefree(*input);
-
-      if(curlx_dyn_addn(&dyn, "/", 1))
-        return CURLE_OUT_OF_MEMORY;
-
-      *input = curlx_dyn_ptr(&dyn);
-    }
-  }
-
-  return CURLE_OK;
+  size_t len = strlen(input);
+  return (len && input[len - 1] == '/');
 }
 
 static char *ipfs_gateway(void)
 {
-  char *ipfs_path = NULL;
-  char *gateway_composed_file_path = NULL;
-  FILE *gateway_file = NULL;
-  char *gateway = curl_getenv("IPFS_GATEWAY");
+  char *ipfs_path_c = NULL;
+  char *gateway_composed_c = NULL;
+  FILE *gfile = NULL;
+  char *gateway_env = getenv("IPFS_GATEWAY");
 
-  /* Gateway is found from environment variable. */
-  if(gateway) {
-    if(ensure_trailing_slash(&gateway))
-      goto fail;
-    return gateway;
-  }
+  if(gateway_env)
+    return strdup(gateway_env);
 
   /* Try to find the gateway in the IPFS data folder. */
-  ipfs_path = curl_getenv("IPFS_PATH");
+  ipfs_path_c = curl_getenv("IPFS_PATH");
 
-  if(!ipfs_path) {
+  if(!ipfs_path_c) {
     char *home = getenv("HOME");
+    /* fallback to "~/.ipfs", as that is the default location. */
     if(home && *home)
-      ipfs_path = aprintf("%s/.ipfs/", home);
-    /* fallback to "~/.ipfs", as that's the default location. */
+      ipfs_path_c = curl_maprintf("%s/.ipfs/", home);
+    if(!ipfs_path_c)
+      goto fail;
   }
 
-  if(!ipfs_path || ensure_trailing_slash(&ipfs_path))
+  gateway_composed_c =
+    curl_maprintf("%s%sgateway", ipfs_path_c,
+                  has_trailing_slash(ipfs_path_c) ? "" : "/");
+
+  if(!gateway_composed_c)
     goto fail;
 
-  gateway_composed_file_path = aprintf("%sgateway", ipfs_path);
+  gfile = curlx_fopen(gateway_composed_c, FOPEN_READTEXT);
+  curl_free(gateway_composed_c);
 
-  if(!gateway_composed_file_path)
-    goto fail;
-
-  gateway_file = fopen(gateway_composed_file_path, FOPEN_READTEXT);
-  Curl_safefree(gateway_composed_file_path);
-
-  if(gateway_file) {
+  if(gfile) {
     int c;
-    struct curlx_dynbuf dyn;
+    struct dynbuf dyn;
+    char *gateway = NULL;
     curlx_dyn_init(&dyn, MAX_GATEWAY_URL_LEN);
 
     /* get the first line of the gateway file, ignore the rest */
-    while((c = getc(gateway_file)) != EOF && c != '\n' && c != '\r') {
+    while((c = getc(gfile)) != EOF && c != '\n' && c != '\r') {
       char c_char = (char)c;
       if(curlx_dyn_addn(&dyn, &c_char, 1))
         goto fail;
     }
 
-    fclose(gateway_file);
-    gateway_file = NULL;
-
     if(curlx_dyn_len(&dyn))
       gateway = curlx_dyn_ptr(&dyn);
 
-    if(gateway)
-      ensure_trailing_slash(&gateway);
-
-    if(!gateway)
-      goto fail;
-
-    Curl_safefree(ipfs_path);
+    curl_free(ipfs_path_c);
+    curlx_fclose(gfile);
 
     return gateway;
   }
 fail:
-  if(gateway_file)
-    fclose(gateway_file);
-  Curl_safefree(gateway);
-  Curl_safefree(ipfs_path);
+  if(gfile)
+    curlx_fclose(gfile);
+  curl_free(ipfs_path_c);
   return NULL;
 }
 
 /*
- * Rewrite ipfs://<cid> and ipns://<cid> to a HTTP(S)
+ * Rewrite ipfs://<cid> and ipns://<cid> to an HTTP(S)
  * URL that can be handled by an IPFS gateway.
  */
 CURLcode ipfs_url_rewrite(CURLU *uh, const char *protocol, char **url,
@@ -162,23 +128,16 @@ CURLcode ipfs_url_rewrite(CURLU *uh, const char *protocol, char **url,
     goto clean;
 
   /* We might have a --ipfs-gateway argument. Check it first and use it. Error
-   * if we do have something but if it's an invalid url.
+   * if we do have something but if it is an invalid url.
    */
   if(config->ipfs_gateway) {
-    /* ensure the gateway ends in a trailing / */
-    if(ensure_trailing_slash(&config->ipfs_gateway) != CURLE_OK) {
-      result = CURLE_OUT_OF_MEMORY;
-      goto clean;
-    }
-
     if(!curl_url_set(gatewayurl, CURLUPART_URL, config->ipfs_gateway,
-                    CURLU_GUESS_SCHEME)) {
+                     CURLU_GUESS_SCHEME)) {
       gateway = strdup(config->ipfs_gateway);
       if(!gateway) {
         result = CURLE_URL_MALFORMAT;
         goto clean;
       }
-
     }
     else {
       result = CURLE_BAD_FUNCTION_ARGUMENT;
@@ -186,7 +145,6 @@ CURLcode ipfs_url_rewrite(CURLU *uh, const char *protocol, char **url,
     }
   }
   else {
-    /* this is ensured to end in a trailing / within ipfs_gateway() */
     gateway = ipfs_gateway();
     if(!gateway) {
       result = CURLE_FILE_COULDNT_READ_FILE;
@@ -218,7 +176,9 @@ CURLcode ipfs_url_rewrite(CURLU *uh, const char *protocol, char **url,
   }
 
   curl_url_get(gatewayurl, CURLUPART_PORT, &gwport, CURLU_URLDECODE);
-  curl_url_get(gatewayurl, CURLUPART_PATH, &gwpath, CURLU_URLDECODE);
+
+  if(curl_url_get(gatewayurl, CURLUPART_PATH, &gwpath, CURLU_URLDECODE))
+    goto clean;
 
   /* get the path from user input */
   curl_url_get(uh, CURLUPART_PATH, &inputpath, CURLU_URLDECODE);
@@ -234,11 +194,10 @@ CURLcode ipfs_url_rewrite(CURLU *uh, const char *protocol, char **url,
   if(inputpath && (inputpath[0] == '/') && !inputpath[1])
     *inputpath = '\0';
 
-  /* ensure the gateway path ends with a trailing slash */
-  ensure_trailing_slash(&gwpath);
-
-  pathbuffer = aprintf("%s%s/%s%s", gwpath, protocol, cid,
-                       inputpath ? inputpath : "");
+  pathbuffer = curl_maprintf("%s%s%s/%s%s", gwpath,
+                             has_trailing_slash(gwpath) ? "" : "/",
+                             protocol, cid,
+                             inputpath ? inputpath : "");
   if(!pathbuffer) {
     goto clean;
   }
@@ -248,7 +207,7 @@ CURLcode ipfs_url_rewrite(CURLU *uh, const char *protocol, char **url,
   }
 
   /* Free whatever it has now, rewriting is next */
-  Curl_safefree(*url);
+  tool_safefree(*url);
 
   if(curl_url_get(uh, CURLUPART_URL, &cloneurl, CURLU_URLENCODE)) {
     goto clean;
@@ -275,13 +234,13 @@ clean:
   {
     switch(result) {
     case CURLE_URL_MALFORMAT:
-      helpf(tool_stderr, "malformed target URL");
+      helpf("malformed target URL");
       break;
     case CURLE_FILE_COULDNT_READ_FILE:
-      helpf(tool_stderr, "IPFS automatic gateway detection failed");
+      helpf("IPFS automatic gateway detection failed");
       break;
     case CURLE_BAD_FUNCTION_ARGUMENT:
-      helpf(tool_stderr, "--ipfs-gateway was given a malformed URL");
+      helpf("--ipfs-gateway was given a malformed URL");
       break;
     default:
       break;
@@ -289,3 +248,4 @@ clean:
   }
   return result;
 }
+#endif /* !CURL_DISABLE_IPFS */
