@@ -106,7 +106,7 @@ static CURLcode test_ws_data_m2_echo(const char *url,
   curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
   curl_easy_setopt(curl, CURLOPT_CONNECT_ONLY, 2L); /* websocket style */
   r = curl_easy_perform(curl);
-  curl_mfprintf(stderr, "curl_easy_perform() returned %u\n", (int)r);
+  curl_mfprintf(stderr, "curl_easy_perform() returned %u\n", r);
   if(r != CURLE_OK)
     goto out;
 
@@ -190,7 +190,7 @@ out:
 }
 
 struct test_ws_m1_ctx {
-  CURL *easy;
+  CURL *curl;
   char *send_buf;
   char *recv_buf;
   size_t send_len, nsent;
@@ -216,7 +216,7 @@ static size_t test_ws_data_m1_read(char *buf, size_t nitems, size_t buflen,
     goto out;
 
   if(!ctx->frame_reading) {
-    curl_ws_start_frame(ctx->easy, CURLWS_BINARY, ctx->send_len);
+    curl_ws_start_frame(ctx->curl, CURLWS_BINARY, ctx->send_len);
     ctx->frame_reading = TRUE;
   }
 
@@ -310,8 +310,8 @@ static CURLcode test_ws_data_m1_echo(const char *url,
     goto out;
   }
 
-  m1_ctx.easy = curl_easy_init();
-  if(!m1_ctx.easy) {
+  m1_ctx.curl = curl_easy_init();
+  if(!m1_ctx.curl) {
     r = CURLE_OUT_OF_MEMORY;
     goto out;
   }
@@ -330,22 +330,22 @@ static CURLcode test_ws_data_m1_echo(const char *url,
     m1_ctx.frames_read = 0;
     m1_ctx.frames_written = 0;
     memset(m1_ctx.recv_buf, 0, plen_max);
-    curl_easy_pause(m1_ctx.easy, CURLPAUSE_CONT);
+    curl_easy_pause(m1_ctx.curl, CURLPAUSE_CONT);
 
-    curl_easy_reset(m1_ctx.easy);
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_URL, url);
+    curl_easy_reset(m1_ctx.curl);
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_URL, url);
     /* use the callback style */
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_USERAGENT, "ws-data");
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_VERBOSE, 1L);
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_USERAGENT, "ws-data");
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_VERBOSE, 1L);
     /* we want to send */
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_UPLOAD, 1L);
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_READFUNCTION, test_ws_data_m1_read);
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_READDATA, &m1_ctx);
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_WRITEFUNCTION,
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_UPLOAD, 1L);
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_READFUNCTION, test_ws_data_m1_read);
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_READDATA, &m1_ctx);
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_WRITEFUNCTION,
                      test_ws_data_m1_write);
-    curl_easy_setopt(m1_ctx.easy, CURLOPT_WRITEDATA, &m1_ctx);
+    curl_easy_setopt(m1_ctx.curl, CURLOPT_WRITEDATA, &m1_ctx);
 
-    curl_multi_add_handle(multi, m1_ctx.easy);
+    curl_multi_add_handle(multi, m1_ctx.curl);
 
     while(1) {
       int still_running; /* keep number of running handles */
@@ -366,7 +366,7 @@ static CURLcode test_ws_data_m1_echo(const char *url,
 
     }
 
-    curl_multi_remove_handle(multi, m1_ctx.easy);
+    curl_multi_remove_handle(multi, m1_ctx.curl);
 
     /* check results */
     if(m1_ctx.frames_read < m1_ctx.nframes) {
@@ -386,8 +386,8 @@ static CURLcode test_ws_data_m1_echo(const char *url,
 out:
   if(multi)
     curl_multi_cleanup(multi);
-  if(m1_ctx.easy) {
-    curl_easy_cleanup(m1_ctx.easy);
+  if(m1_ctx.curl) {
+    curl_easy_cleanup(m1_ctx.curl);
   }
   free(m1_ctx.send_buf);
   free(m1_ctx.recv_buf);
@@ -411,7 +411,7 @@ static void test_ws_data_usage(const char *msg)
 static CURLcode test_cli_ws_data(const char *URL)
 {
 #ifndef CURL_DISABLE_WEBSOCKETS
-  CURLcode res = CURLE_OK;
+  CURLcode result = CURLE_OK;
   const char *url;
   size_t plen_min = 0, plen_max = 0, count = 1;
   int ch, model = 2;
@@ -428,21 +428,19 @@ static CURLcode test_cli_ws_data(const char *URL)
       break;
     case 'h':
       test_ws_data_usage(NULL);
-      res = CURLE_BAD_FUNCTION_ARGUMENT;
-      goto cleanup;
+      return CURLE_BAD_FUNCTION_ARGUMENT;
     case 'c':
-      count = (size_t)strtol(coptarg, NULL, 10);
+      count = (size_t)atol(coptarg);
       break;
     case 'm':
-      plen_min = (size_t)strtol(coptarg, NULL, 10);
+      plen_min = (size_t)atol(coptarg);
       break;
     case 'M':
-      plen_max = (size_t)strtol(coptarg, NULL, 10);
+      plen_max = (size_t)atol(coptarg);
       break;
     default:
       test_ws_data_usage("invalid option");
-      res = CURLE_BAD_FUNCTION_ARGUMENT;
-      goto cleanup;
+      return CURLE_BAD_FUNCTION_ARGUMENT;
     }
   }
   test_argc -= coptind;
@@ -454,27 +452,28 @@ static CURLcode test_cli_ws_data(const char *URL)
   if(plen_max < plen_min) {
     curl_mfprintf(stderr, "maxlen must be >= minlen, got %zu-%zu\n",
                   plen_min, plen_max);
-    res = CURLE_BAD_FUNCTION_ARGUMENT;
-    goto cleanup;
+    return CURLE_BAD_FUNCTION_ARGUMENT;
   }
 
   if(test_argc != 1) {
     test_ws_data_usage(NULL);
-    res = CURLE_BAD_FUNCTION_ARGUMENT;
-    goto cleanup;
+    return CURLE_BAD_FUNCTION_ARGUMENT;
   }
   url = test_argv[0];
 
-  curl_global_init(CURL_GLOBAL_ALL);
+  if(curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
+    curl_mfprintf(stderr, "curl_global_init() failed\n");
+    return (CURLcode)3;
+  }
 
   if(model == 1)
-    res = test_ws_data_m1_echo(url, plen_min, plen_max);
+    result = test_ws_data_m1_echo(url, plen_min, plen_max);
   else
-    res = test_ws_data_m2_echo(url, count, plen_min, plen_max);
+    result = test_ws_data_m2_echo(url, count, plen_min, plen_max);
 
-cleanup:
   curl_global_cleanup();
-  return res;
+
+  return result;
 
 #else /* !CURL_DISABLE_WEBSOCKETS */
   (void)URL;
