@@ -37,8 +37,8 @@
  */
 static CURLcode test_lib560(const char *URL)
 {
-  CURL *http_handle = NULL;
-  CURLM *multi_handle = NULL;
+  CURL *curl = NULL;
+  CURLM *multi = NULL;
   CURLcode res = CURLE_OK;
 
   int still_running; /* keep number of running handles */
@@ -49,22 +49,22 @@ static CURLcode test_lib560(const char *URL)
   ** curl_global_init called indirectly from curl_easy_init.
   */
 
-  easy_init(http_handle);
+  easy_init(curl);
 
   /* set options */
-  easy_setopt(http_handle, CURLOPT_URL, URL);
-  easy_setopt(http_handle, CURLOPT_HEADER, 1L);
-  easy_setopt(http_handle, CURLOPT_SSL_VERIFYPEER, 0L);
-  easy_setopt(http_handle, CURLOPT_SSL_VERIFYHOST, 0L);
+  easy_setopt(curl, CURLOPT_URL, URL);
+  easy_setopt(curl, CURLOPT_HEADER, 1L);
+  easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
+  easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
 
   /* init a multi stack */
-  multi_init(multi_handle);
+  multi_init(multi);
 
   /* add the individual transfers */
-  multi_add_handle(multi_handle, http_handle);
+  multi_add_handle(multi, curl);
 
   /* we start some action by calling perform right away */
-  multi_perform(multi_handle, &still_running);
+  multi_perform(multi, &still_running);
 
   abort_on_test_timeout();
 
@@ -85,7 +85,7 @@ static CURLcode test_lib560(const char *URL)
     timeout.tv_usec = 0;
 
     /* get file descriptors from the transfers */
-    multi_fdset(multi_handle, &fdread, &fdwrite, &fdexcep, &maxfd);
+    multi_fdset(multi, &fdread, &fdwrite, &fdexcep, &maxfd);
 
     /* At this point, maxfd is guaranteed to be greater or equal than -1. */
 
@@ -94,7 +94,7 @@ static CURLcode test_lib560(const char *URL)
     abort_on_test_timeout();
 
     /* timeout or readable/writable sockets */
-    multi_perform(multi_handle, &still_running);
+    multi_perform(multi, &still_running);
 
     abort_on_test_timeout();
   }
@@ -103,8 +103,8 @@ test_cleanup:
 
   /* undocumented cleanup sequence - type UA */
 
-  curl_multi_cleanup(multi_handle);
-  curl_easy_cleanup(http_handle);
+  curl_multi_cleanup(multi);
+  curl_easy_cleanup(curl);
   curl_global_cleanup();
 
   return res;
