@@ -63,8 +63,8 @@ static CURLcode test_lib654(const char *URL)
 {
   static const char testdata[] = "dummy\n";
 
-  CURL *easy = NULL;
-  CURL *easy2 = NULL;
+  CURL *curl = NULL;
+  CURL *curl2 = NULL;
   curl_mime *mime = NULL;
   curl_mimepart *part;
   struct curl_slist *hdrs = NULL;
@@ -81,16 +81,16 @@ static CURLcode test_lib654(const char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  easy = curl_easy_init();
+  curl = curl_easy_init();
 
   /* First set the URL that is about to receive our POST. */
-  test_setopt(easy, CURLOPT_URL, URL);
+  test_setopt(curl, CURLOPT_URL, URL);
 
   /* get verbose debug output please */
-  test_setopt(easy, CURLOPT_VERBOSE, 1L);
+  test_setopt(curl, CURLOPT_VERBOSE, 1L);
 
   /* include headers in the output */
-  test_setopt(easy, CURLOPT_HEADER, 1L);
+  test_setopt(curl, CURLOPT_HEADER, 1L);
 
   /* Prepare the callback structure. */
   pooh.readptr = testdata;
@@ -98,7 +98,7 @@ static CURLcode test_lib654(const char *URL)
   pooh.freecount = 0;
 
   /* Build the mime tree. */
-  mime = curl_mime_init(easy);
+  mime = curl_mime_init(curl);
   part = curl_mime_addpart(mime);
   curl_mime_data(part, "hello", CURL_ZERO_TERMINATED);
   curl_mime_name(part, "greeting");
@@ -113,11 +113,11 @@ static CURLcode test_lib654(const char *URL)
                     free_callback, &pooh);
 
   /* Bind mime data to its easy handle. */
-  test_setopt(easy, CURLOPT_MIMEPOST, mime);
+  test_setopt(curl, CURLOPT_MIMEPOST, mime);
 
   /* Duplicate the handle. */
-  easy2 = curl_easy_duphandle(easy);
-  if(!easy2) {
+  curl2 = curl_easy_duphandle(curl);
+  if(!curl2) {
     curl_mfprintf(stderr, "curl_easy_duphandle() failed\n");
     res = TEST_ERR_FAILURE;
     goto test_cleanup;
@@ -129,7 +129,7 @@ static CURLcode test_lib654(const char *URL)
   mime = NULL;  /* Already cleaned up. */
 
   /* Perform on the first handle: should not send any data. */
-  res = curl_easy_perform(easy);
+  res = curl_easy_perform(curl);
   if(res != CURLE_OK) {
     curl_mfprintf(stderr, "curl_easy_perform(original) failed\n");
     goto test_cleanup;
@@ -137,7 +137,7 @@ static CURLcode test_lib654(const char *URL)
 
   /* Perform on the second handle: if the bound mime structure has not been
      duplicated properly, it should cause a valgrind error. */
-  res = curl_easy_perform(easy2);
+  res = curl_easy_perform(curl2);
   if(res != CURLE_OK) {
     curl_mfprintf(stderr, "curl_easy_perform(duplicated) failed\n");
     goto test_cleanup;
@@ -146,8 +146,8 @@ static CURLcode test_lib654(const char *URL)
   /* Free the duplicated handle: it should call free_callback again.
      If the mime copy was bad or not automatically released, valgrind
      will signal it. */
-  curl_easy_cleanup(easy2);
-  easy2 = NULL;  /* Already cleaned up. */
+  curl_easy_cleanup(curl2);
+  curl2 = NULL;  /* Already cleaned up. */
 
   if(pooh.freecount != 2) {
     curl_mfprintf(stderr, "free_callback() called %d times instead of 2\n",
@@ -157,8 +157,8 @@ static CURLcode test_lib654(const char *URL)
   }
 
 test_cleanup:
-  curl_easy_cleanup(easy);
-  curl_easy_cleanup(easy2);
+  curl_easy_cleanup(curl);
+  curl_easy_cleanup(curl2);
   curl_mime_free(mime);
   curl_global_cleanup();
   return res;
