@@ -27,11 +27,18 @@ package processhelp;
 use strict;
 use warnings;
 
+use Time::HiRes;
+
+use pathhelp qw(
+    os_is_win
+    );
+
+my $has_win32_process;
+
 BEGIN {
     use base qw(Exporter);
 
     our @EXPORT = qw(
-        portable_sleep
         pidfromfile
         pidexists
         pidwait
@@ -43,15 +50,16 @@ BEGIN {
         clear_advisor_read_lock
     );
 
-    # portable sleeping needs Time::HiRes
-    eval {
-        no warnings "all";
-        require Time::HiRes;
-    };
-    # portable sleeping falls back to native Sleep on Windows
-    eval {
-        no warnings "all";
-        require Win32;
+    if(os_is_win() && $^O ne 'MSWin32') {
+        $has_win32_process = eval {
+            no warnings "all";
+            # https://metacpan.org/pod/Win32::Process
+            require Win32::Process;
+            # https://metacpan.org/pod/Win32::Process::List
+            require Win32::Process::List;
+        };
+    } else {
+      $has_win32_process = 0;
     }
 }
 
@@ -61,31 +69,10 @@ use serverhelp qw(
     datasockf_pidfilename
     );
 
-use pathhelp qw(
-    os_is_win
+use globalconfig qw(
+    $dev_null
     );
 
-#######################################################################
-# portable_sleep uses Time::HiRes::sleep if available and falls back
-# to the classic approach of using select(undef, undef, undef, ...).
-# even though that one is not portable due to being implemented using
-# select on Windows: https://perldoc.perl.org/perlport.html#select
-# Therefore it uses Win32::Sleep on Windows systems instead.
-#
-sub portable_sleep {
-    my ($seconds) = @_;
-
-    if($Time::HiRes::VERSION) {
-        Time::HiRes::sleep($seconds);
-    }
-    elsif (os_is_win()) {
-        Win32::Sleep($seconds*1000);
-    }
-    else {
-        select(undef, undef, undef, $seconds);
-    }
-}
-
 #######################################################################
 # pidfromfile returns the pid stored in the given pidfile.  The value
 # of the returned pid will never be a negative value. It will be zero
@@ -94,12 +81,18 @@ sub portable_sleep {
 #
 sub pidfromfile {
     my $pidfile = $_[0];
+    my $timeout_sec = $_[1];
     my $pid = 0;
-
-    if(-f $pidfile && -s $pidfile && open(my $pidfh, "<", "$pidfile")) {
-        $pid = 0 + <$pidfh>;
-        close($pidfh);
-        $pid = 0 if($pid < 0);
+    my $waits = 0;
+    # wait at max 15 seconds for the file to exist and have valid content
+    while(!$pid && ($waits <= ($timeout_sec * 10))) {
+        if(-f $pidfile && -s $pidfile && open(my $pidfh, "<", "$pidfile")) {
+            $pid = 0 + <$pidfh>;
+            close($pidfh);
+            $pid = 0 if($pid < 0);
+        }
+        Time::HiRes::sleep(0.1) unless $pid || !$timeout_sec;
+        ++$waits;
     }
     return $pid;
 }
@@ -109,8 +102,8 @@ sub pidfromfile {
 #
 sub winpid_to_pid {
     my $vpid = $_[0];
-    if(($^O eq 'cygwin' || $^O eq 'msys') && $vpid > 65536) {
-        my $pid = Cygwin::winpid_to_pid($vpid - 65536);
+    if(($^O eq 'cygwin' || $^O eq 'msys') && $vpid > 4194304) {
+        my $pid = Cygwin::winpid_to_pid($vpid - 4194304);
         if($pid) {
             return $pid;
         } else {
@@ -132,13 +125,21 @@ sub pidexists {
     if($pid > 0) {
         # verify if currently existing Windows process
         $pid = winpid_to_pid($pid);
-        if ($pid > 65536 && os_is_win()) {
-            $pid -= 65536;
+        if($pid > 4194304 && os_is_win()) {
+            $pid -= 4194304;
             if($^O ne 'MSWin32') {
-                my $filter = "PID eq $pid";
-                my $result = `tasklist -fi \"$filter\" 2>nul`;
-                if(index($result, "$pid") != -1) {
-                    return -$pid;
+                if($has_win32_process) {
+                    my %processes = Win32::Process::List->new()->GetProcesses();
+                    if(exists $processes{$pid}) {
+                        return -$pid;
+                    }
+                } else {
+                    my $filter = "PID eq $pid";
+                    # https://ss64.com/nt/tasklist.html
+                    my $result = `tasklist -fi \"$filter\" 2>$dev_null`;
+                    if(index($result, "$pid") != -1) {
+                        return -$pid;
+                    }
                 }
                 return 0;
             }
@@ -162,13 +163,16 @@ sub pidterm {
     if($pid > 0) {
         # request the process to quit
         $pid = winpid_to_pid($pid);
-        if ($pid > 65536 && os_is_win()) {
-            $pid -= 65536;
+        if($pid > 4194304 && os_is_win()) {
+            $pid -= 4194304;
             if($^O ne 'MSWin32') {
-                my $filter = "PID eq $pid";
-                my $result = `tasklist -fi \"$filter\" 2>nul`;
-                if(index($result, "$pid") != -1) {
-                    system("taskkill -fi \"$filter\" >nul 2>&1");
+                if($has_win32_process) {
+                    Win32::Process::KillProcess($pid, 0);
+                } else {
+                    # https://ss64.com/nt/taskkill.html
+                    my $cmd = "taskkill -f -t -pid $pid >$dev_null 2>&1";
+                    print "Executing: '$cmd'\n";
+                    system($cmd);
                 }
                 return;
             }
@@ -188,15 +192,16 @@ sub pidkill {
     if($pid > 0) {
         # request the process to quit
         $pid = winpid_to_pid($pid);
-        if ($pid > 65536 && os_is_win()) {
-            $pid -= 65536;
+        if($pid > 4194304 && os_is_win()) {
+            $pid -= 4194304;
             if($^O ne 'MSWin32') {
-                my $filter = "PID eq $pid";
-                my $result = `tasklist -fi \"$filter\" 2>nul`;
-                if(index($result, "$pid") != -1) {
-                    system("taskkill -f -t -fi \"$filter\" >nul 2>&1");
-                    # Windows XP Home compatibility
-                    system("tskill $pid >nul 2>&1");
+                if($has_win32_process) {
+                    Win32::Process::KillProcess($pid, 0);
+                } else {
+                    # https://ss64.com/nt/taskkill.html
+                    my $cmd = "taskkill -f -t -pid $pid >$dev_null 2>&1";
+                    print "Executing: '$cmd'\n";
+                    system($cmd);
                 }
                 return;
             }
@@ -216,12 +221,22 @@ sub pidwait {
 
     $pid = winpid_to_pid($pid);
     # check if the process exists
-    if ($pid > 65536 && os_is_win()) {
+    if($pid > 4194304 && os_is_win()) {
         if($flags == &WNOHANG) {
             return pidexists($pid)?0:$pid;
         }
+        my $start = time;
+        my $warn_at = 5;
         while(pidexists($pid)) {
-            portable_sleep(0.01);
+            if(time - $start > $warn_at) {
+                print "pidwait: still waiting for PID ", $pid, "\n";
+                $warn_at += 5;
+                if($warn_at > 20) {
+                    print "pidwait: giving up waiting for PID ", $pid, "\n";
+                    last;
+                }
+            }
+            Time::HiRes::sleep(0.2);
         }
         return $pid;
     }
@@ -243,7 +258,7 @@ sub processexists {
     my $pidfile = $_[0];
 
     # fetch pid from pidfile
-    my $pid = pidfromfile($pidfile);
+    my $pid = pidfromfile($pidfile, 0);
 
     if($pid > 0) {
         # verify if currently alive
@@ -252,7 +267,7 @@ sub processexists {
         }
         else {
             # get rid of the certainly invalid pidfile
-            unlink($pidfile) if($pid == pidfromfile($pidfile));
+            unlink($pidfile) if($pid == pidfromfile($pidfile, 0));
             # reap its dead children, if not done yet
             pidwait($pid, &WNOHANG);
             # negative return value means dead process
@@ -327,7 +342,9 @@ sub killpid {
                 }
             }
             last if(not scalar(@signalled));
-            portable_sleep(0.05);
+            # give any zombies of us a chance to move on to the afterlife
+            pidwait(0, &WNOHANG);
+            Time::HiRes::sleep(0.05);
         }
     }
 
